package net.pingex.dcf.commands.permissions;

import net.pingex.dcf.commands.Command;
import net.pingex.dcf.core.Configuration;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.mapdb.DB;
import org.mapdb.DBException;
import org.mapdb.DBMaker;
import org.mapdb.Serializer;
import sx.blah.discord.handle.obj.IGuild;
import sx.blah.discord.handle.obj.IRole;
import sx.blah.discord.handle.obj.IUser;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * The one and only permissions provider for now.
 */
public class DefaultPermissionsProvider implements ICommandPermissionsProvider
{
	/**
	 * Datastore instance
	 */
	private DB datastore;

	/**
	 * User permissions store
	 */
	private Map<String, Boolean> userStore;

	/**
	 * Group permissions store
	 */
	private Map<String, Boolean> groupStore;

	private static final Logger LOGGER = LogManager.getLogger(DefaultPermissionsProvider.class);

	public DefaultPermissionsProvider()
	{
		LOGGER.info("Building datastore.");

		try
		{
			datastore = DBMaker.fileDB(new File(Configuration.DATA_DIR + "/permissions.db"))
					.closeOnJvmShutdown()
					.make();

			userStore = datastore.hashMap("user", Serializer.STRING, Serializer.BOOLEAN).createOrOpen();
			groupStore = datastore.hashMap("group", Serializer.STRING, Serializer.BOOLEAN).createOrOpen();
		}
		catch(DBException e)
		{
			LOGGER.error("Error setting up the datastore.", e);
			LOGGER.error("Switching to local memory storage. PERMISSIONS ARE NOT LOADED AND CHANGES WON'T BE SAVED.");
			userStore = new HashMap<>();
			groupStore = new HashMap<>();
		}
	}

	@Override
	public Boolean validateUser(IGuild guild, IUser user, Command command)
	{
		return userStore.get(generateUserKey(guild, user, command));
	}

	@Override
	public Boolean validateGroup(IRole role, Command command)
	{
		return groupStore.get(generateGroupKey(role, command));
	}

	@Override
	public void setUserPermission(IGuild guild, IUser user, Command command, Boolean value)
	{
		if(value == null) // Unset
			userStore.remove(generateUserKey(guild, user, command));
		else
			userStore.put(generateUserKey(guild, user, command), value);
	}

	@Override
	public void setGroupPermissions(IRole role, Command command, Boolean value)
	{
		if(value == null) // Unset
			groupStore.remove(generateGroupKey(role, command));
		else
			groupStore.put(generateGroupKey(role, command), value);
	}

	private String generateUserKey(IGuild guild, IUser user, Command command)
	{
		return (guild != null ? guild.getID() : "*") + "/" + user.getID() + "/" + command.getName();
	}

	private String generateGroupKey(IRole role, Command command)
	{
		return role.getID() + "/" + command.getName();
	}
}
