package net.pingex.dcf.commands.permissions.audit;

import net.pingex.dcf.commands.Context;
import net.pingex.dcf.commands.audit.AuditResult;
import net.pingex.dcf.commands.audit.IAuditComponentProvider;
import net.pingex.dcf.commands.permissions.ICommandPermissionsProvider;

/**
 * This class checks permission against a rule defined for a specific user globally.
 */
public class UserGlobalCheck implements IAuditComponentProvider
{
	/**
	 * Permissions provider to use
	 */
	private ICommandPermissionsProvider provider;

	/**
	 * Provider for this object to use
	 * @param provider Permissions provider this object will use
	 */
	public UserGlobalCheck(ICommandPermissionsProvider provider)
	{
		this.provider = provider;
	}

	@Override
	public AuditResult doAudit(Context context)
	{
		Boolean returnedValue = provider.validateUser(null, context.getUser(), context.getCommand());

		if(returnedValue == null) // No rule for user
			return new AuditResult(AuditResult.ResultCode.NOOP, "No global rule for this user.");
		else if(returnedValue) // Rule says yes
			return new AuditResult(AuditResult.ResultCode.PASS);
		else // Rule says no
			return new AuditResult(AuditResult.ResultCode.FAIL, "Global rule denied access for this user.");
	}

	@Override
	public String name()
	{
		return "User global check";
	}

	@Override
	public String description()
	{
		return "Checks against a global rule defined for a specific user.";
	}

	@Override
	public int priority()
	{
		return 2;
	}
}
