package net.pingex.dcf.core;

import org.apache.commons.configuration2.ConfigurationUtils;
import org.apache.commons.configuration2.FileBasedConfiguration;
import org.apache.commons.configuration2.ImmutableConfiguration;
import org.apache.commons.configuration2.PropertiesConfiguration;
import org.apache.commons.configuration2.builder.FileBasedConfigurationBuilder;
import org.apache.commons.configuration2.builder.fluent.Parameters;
import org.apache.commons.configuration2.ex.ConfigurationException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.io.File;

/**
 * Configuration manager
 */
public class Configuration
{
	/**
	 * File name to look for
	 */
	private static final String FILENAME = "dcf.properties";

	/**
	 * Logger
	 */
	private static final Logger LOGGER = LogManager.getLogger(Configuration.class);

	/**
	 * Main configuration store
	 */
	private static org.apache.commons.configuration2.Configuration store;

	/**
	 * Load configuration file
	 */
	public static void load()
	{
		try
		{
			LOGGER.info("Loading configuration file");
			store = new FileBasedConfigurationBuilder<FileBasedConfiguration>(PropertiesConfiguration.class)
					.configure(new Parameters().fileBased().setFileName(FILENAME))
					.getConfiguration();
		} catch (ConfigurationException e)
		{
			LOGGER.fatal("Failed to load configuration", e);
		}
	}

	/**
	 * Get the store in read-only mode
	 * @return The read-only store
	 */
	public static ImmutableConfiguration getStore()
	{
		return ConfigurationUtils.unmodifiableConfiguration(store);
	}

	// ========================================

	/**
	 * Bot display name
	 */
	public static String BOT_NAME = "DCF-enabled Discord Bot";

	/**
	 * Initial connection token
	 */
	public static String CONNECTION_TOKEN = null;

	/**
	 * Initial connection username
	 */
	public static String CONNECTION_USERNAME = null;

	/**
	 * Initial connection password
	 */
	public static String CONNECTION_PASSWORD = null;

	/**
	 * Plugins directory
	 */
	public static String PLUGINS_DIR = "plugins";

	/**
	 * Data storage directory
	 */
	public static String DATA_DIR = "data";

	/**
	 * Command prefix string
	 */
	public static String COMMAND_PREFIX = "!";

	/**
	 *
	 */
	public static String BOT_OWNER = "";

	/**
	 * Tells if the bot is configured to connect using a token or an username/password tuple.
	 * @return Whether the main connection is a bot, or not
	 */
	public static boolean isConnectionToken()
	{
		return store.containsKey("discord.token");
	}

	/**
	 * Load config keys
	 */
	public static void init()
	{
		BOT_NAME = store.getString("general.bot_name", BOT_NAME);
		PLUGINS_DIR = store.getString("general.plugins_dir", PLUGINS_DIR);
		DATA_DIR = store.getString("general.storage_dir", DATA_DIR);
		COMMAND_PREFIX = store.getString("commands.prefix", COMMAND_PREFIX);
		BOT_OWNER = store.getString("general.bot_owner", BOT_OWNER);

		// Validate main connection username/password or token
		if(isConnectionToken())
			CONNECTION_TOKEN = store.getString("discord.token"); // We imply that the token exists, as its existence is already checked in isConnectionToken()
		else
			if(!store.containsKey("discord.username") || !store.containsKey("discord.password"))
			{
				LOGGER.error("Connection username/password not specified !");
			}
			else
			{
				CONNECTION_USERNAME = store.getString("discord.username");
				CONNECTION_PASSWORD = store.getString("discord.password");
			}

		// Validate plugins directory
		File testingPath = new File(PLUGINS_DIR);
		if(!testingPath.exists()) LOGGER.error("Specified plugins path doesn't exist.");
		else if(!testingPath.isDirectory()) LOGGER.error("Specified plugins directory is not a directory.");
	}
}
